@@ -26,7 +26,10 @@ type 'a mappable = 'a t
 
 include Array
 
-let map = map
+let singleton x = [|x|]
+(*$Q singleton
+  Q.int (fun x -> let s = singleton x in s.(0) = x && length s = 1)
+*)
 
 let modify f a =
   for i = 0 to length a - 1 do
@@ -41,8 +44,8 @@ let modifyi f a =
 (*$T modify
   let a = [|3;2;1|] in modify (fun x -> x + 1) a; a = [|4;3;2|]
 *)(*$T modifyi
-  let a = [|3;2;1|] in modifyi (fun i x -> i * x) a; a = [|0;2;2|]
-*)
+    let a = [|3;2;1|] in modifyi (fun i x -> i * x) a; a = [|0;2;2|]
+  *)
 
 let fold_lefti f x a =
   let r = ref x in
@@ -248,20 +251,20 @@ let partition p xs =
   let n2 = n - n1 in
   let j = ref 0 in
   let xs1 = init n1
-    (fun _ ->
-       (* Find the next set bit in the BitSet. *)
-       while not (BatBitSet.mem bs !j) do incr j done;
-       let r = xs.(!j) in
-       incr j;
-       r) in
+      (fun _ ->
+        (* Find the next set bit in the BitSet. *)
+        while not (BatBitSet.mem bs !j) do incr j done;
+        let r = xs.(!j) in
+        incr j;
+        r) in
   let j = ref 0 in
   let xs2 = init n2
-    (fun _ ->
-       (* Find the next clear bit in the BitSet. *)
-       while BatBitSet.mem bs !j do incr j done;
-       let r = xs.(!j) in
-       incr j;
-       r) in
+      (fun _ ->
+        (* Find the next clear bit in the BitSet. *)
+        while BatBitSet.mem bs !j do incr j done;
+        let r = xs.(!j) in
+        incr j;
+        r) in
   xs1, xs2
 (*$Q partition
   (Q.pair (Q.array Q.small_int) (Q.fun1 Q.small_int Q.bool)) (fun (a, f) -> \
@@ -276,17 +279,17 @@ let partition p xs =
 let enum xs =
   let rec make start xs =
     let n = length xs in
-      (* inside the loop, as [make] may later be called with another array *)
+    (* inside the loop, as [make] may later be called with another array *)
     BatEnum.make
       ~next:(fun () ->
-	       if !start < n then
-		 xs.(BatRef.post_incr start)
-	       else
-		 raise BatEnum.No_more_elements)
+        if !start < n then
+          xs.(BatRef.post_incr start)
+        else
+          raise BatEnum.No_more_elements)
       ~count:(fun () ->
-		n - !start)
+        n - !start)
       ~clone:(fun () ->
-		make (ref !start) xs)
+        make (BatRef.copy start) xs)
   in
   make (ref 0) xs
 (*$Q enum
@@ -309,15 +312,14 @@ let backwards xs =
   let rec make start xs =
     BatEnum.make
       ~next:(fun () ->
-	       if !start > 0 then
-		 xs.(BatRef.pre_decr start)
-	       else
-		 raise BatEnum.No_more_elements)
+        if !start > 0 then
+          xs.(BatRef.pre_decr start)
+        else
+          raise BatEnum.No_more_elements)
       ~count:(fun () ->
-		!start)
+        !start)
       ~clone:(fun () ->
-		let xs' = Array.sub xs 0 !start in
-		make (BatRef.copy start) xs')
+        make (BatRef.copy start) xs)
   in
   make (ref (length xs)) xs
 (*$Q backwards
@@ -342,9 +344,9 @@ let of_enum e =
   (* This assumes, reasonably, that init traverses the array in order. *)
   Array.init n
     (fun _i ->
-       match BatEnum.get e with
-       | Some x -> x
-       | None -> assert false (*BISECT-VISIT*))
+      match BatEnum.get e with
+      | Some x -> x
+      | None -> assert false (*BISECT-VISIT*))
 
 let of_backwards e =
   of_list (BatList.of_backwards e)
@@ -467,6 +469,25 @@ let map2 f xs ys =
      with Invalid_argument _ -> true
 *)
 
+let cartesian_product a b =
+  let na = length a in
+  let nb = length b in
+  init
+    (na * nb)
+    (fun j -> let i = j / nb in
+              a.(i), b.(j - i*nb))
+
+(*$T cartesian_product
+  let a = cartesian_product [|1;2|] [|"a";"b"|] in \
+    sort Pervasives.compare a; \
+    a = [|1,"a"; 1,"b"; 2,"a"; 2, "b" |]
+*)
+
+(*$Q cartesian_product
+  (Q.pair (Q.array Q.small_int) (Q.array Q.small_int)) \
+    (fun (a,b) -> length (cartesian_product a b) = length a * length b)
+*)
+
 let compare cmp a b =
   let length_a = Array.length a in
   let length_b = Array.length b in
@@ -477,9 +498,9 @@ let compare cmp a b =
       if result = 0 then aux (i + 1)
       else               result
     else
-      if length_a = length_b then	0
-      else if length_a < length_b then -1
-      else                              1
+    if length_a = length_b then	0
+    else if length_a < length_b then -1
+    else                              1
   in
   aux 0
 
@@ -494,17 +515,17 @@ let compare cmp a b =
 
 let print ?(first="[|") ?(last="|]") ?(sep="; ") print_a  out t =
   match length t with
-    | 0 ->
-	BatInnerIO.nwrite out first;
-	BatInnerIO.nwrite out last
-    | n ->
-	BatInnerIO.nwrite out first;
-	print_a out (unsafe_get t 0);
-	for i = 1 to n - 1 do
-	  BatInnerIO.nwrite out sep;
-	  print_a out (unsafe_get t i);
-	done;
-	BatInnerIO.nwrite out last
+  | 0 ->
+    BatInnerIO.nwrite out first;
+    BatInnerIO.nwrite out last
+  | n ->
+    BatInnerIO.nwrite out first;
+    print_a out (unsafe_get t 0);
+    for i = 1 to n - 1 do
+      BatInnerIO.nwrite out sep;
+      print_a out (unsafe_get t i);
+    done;
+    BatInnerIO.nwrite out last
 (*$T
   BatIO.to_string (print ~sep:"," ~first:"[" ~last:"]" BatInt.print) \
     [|2;4;66|] = "[2,4,66]"
@@ -538,21 +559,61 @@ let max a = reduce Pervasives.max a
   min [|1;2;3|] = 1
   min [|2;3;1|] = 1
 *)(*$T max
-  max [|1;2;3|] = 3
-  max [|2;3;1|] = 3
-*)
+    max [|1;2;3|] = 3
+    max [|2;3;1|] = 3
+  *)
 
 let sum = reduce (+)
 let fsum = reduce (+.)
 
 (*$T sum
- sum [|1;2;3|] = 6
- sum [|0|] = 0
+  sum [|1;2;3|] = 6
+  sum [|0|] = 0
 *) (*$T fsum
- fsum [|1.0;2.0;3.0|] = 6.0
- fsum [|0.0|] = 0.0
+     fsum [|1.0;2.0;3.0|] = 6.0
+     fsum [|0.0|] = 0.0
+   *)
+
+let kahan_sum arr =
+  let sum = ref 0. in
+  let err = ref 0. in
+  for i = 0 to Array.length arr - 1 do
+    let x = arr.(i) -. !err in
+    let new_sum = !sum +. x in
+    err := (new_sum -. !sum) -. x;
+    sum := new_sum +. 0.;
+    (* this suspicious +. 0. is added to help
+       the hand of the somewhat flaky unboxing optimizer;
+       it hopefully won't be necessary anymore
+       in a few OCaml versions *)
+  done;
+  !sum +. 0.
+
+(*$T kahan_sum
+   kahan_sum [| |] = 0.
+   kahan_sum [| 1.; 2. |] = 3.
+   let n, x = 1_000, 1.1 in \
+     Float.approx_equal (float n *. x) \
+                        (kahan_sum (Array.make n x))
 *)
 
+let flength a =
+  float_of_int (length a)
+
+let avg a =
+  (float_of_int (sum a)) /. (flength a)
+
+let favg a =
+  (fsum a) /. (flength a)
+;;
+(*$T avg
+  avg [|1; 2; 3|] = 2.
+  avg [|0|] = 0.
+*) (*$T favg
+     favg [|1.0; 2.0; 3.0|] = 2.0
+     favg [|0.0|] = 0.0
+   *)
+
 (* meant for tests, don't care about side effects being repeated
    or not failing early *)
 let is_sorted_by f xs =
@@ -585,6 +646,33 @@ let decorate_fast_sort f xs =
     (fun (a, f) -> is_sorted_by f (decorate_fast_sort f a))
 *)
 
+let bsearch cmp arr x =
+  let rec bsearch i j =
+    if i > j
+      then `Just_after j
+      else
+        let middle = i + (j - i) / 2 in (* avoid overflow *)
+        match cmp x arr.(middle) with
+        | BatOrd.Eq -> `At middle
+        | BatOrd.Lt -> bsearch i (middle - 1)
+        | BatOrd.Gt -> bsearch (middle + 1) j
+  in
+  if length arr = 0 then `Empty
+  else match cmp arr.(0) x, cmp arr.(length arr - 1) x with
+  | BatOrd.Gt, _ -> `All_bigger
+  | _, BatOrd.Lt -> `All_lower
+  | _ -> bsearch 0 (length arr - 1)
+
+(*$T bsearch
+  bsearch BatInt.ord [|1; 2; 2; 3; 4; 10|] 3 = `At 3
+  bsearch BatInt.ord [|1; 2; 2; 3; 4; 10|] 5 = `Just_after 4
+  bsearch BatInt.ord [|1; 2; 5; 5; 11; 12|] 1 = `At 0
+  bsearch BatInt.ord [|1; 2; 5; 5; 11; 12|] 12 = `At 5
+  bsearch BatInt.ord [|1; 2; 2; 3; 4; 9|] 10 = `All_lower
+  bsearch BatInt.ord [|1; 2; 2; 3; 4; 9|] 0 = `All_bigger
+  bsearch BatInt.ord [| |] 3 = `Empty
+*)
+
 let insert xs x i =
   if i > Array.length xs then invalid_arg "Array.insert: offset out of range";
   Array.init (Array.length xs + 1) (fun j -> if j < i then xs.(j) else if j > i then xs.(j-1) else x)
@@ -651,10 +739,10 @@ end
 
 let left a len = if len >= length a then a else sub a 0 len
 let right a len = let alen = length a in
-		  if len >= alen then a else sub a (alen - len) len
+  if len >= alen then a else sub a (alen - len) len
 let head a pos = left a pos
 let tail a pos = let alen = length a in
-		 if pos >= alen then [||] else sub a pos (alen - pos)
+  if pos >= alen then [||] else sub a pos (alen - pos)
 
 (*$= left & ~printer:(IO.to_string (Array.print Int.print))
   (left [|1;2;3|] 1) [|1|]
@@ -663,20 +751,20 @@ let tail a pos = let alen = length a in
   (left [|1;2;3|] 10)[|1;2;3|]
   (left [|1;2;3|] 0) [||]
 *) (*$= right & ~printer:(IO.to_string (Array.print Int.print))
-  (right [|1;2;3|] 1) [|3|]
-  (right [|1;2|] 3) [|1;2|]
-  (right [|1;2;3|] 3) [|1;2;3|]
-  (right [|1;2;3|] 10) [|1;2;3|]
-  (right [|1;2;3|] 0) [||]
-*) (*$= tail & ~printer:(IO.to_string (Array.print Int.print))
-  (tail [|1;2;3|] 1) [|2;3|]
-  [||] (tail [|1;2;3|] 10)
-  (tail [|1;2;3|] 0) [|1;2;3|]
-*) (*$= head & ~printer:(IO.to_string (Array.print Int.print))
-  (head [|1;2;3|] 1) [|1|]
-  (head [|1;2;3|] 10) [|1;2;3|]
-  (head [|1;2;3|] 0) [||]
-*)
+     (right [|1;2;3|] 1) [|3|]
+     (right [|1;2|] 3) [|1;2|]
+     (right [|1;2;3|] 3) [|1;2;3|]
+     (right [|1;2;3|] 10) [|1;2;3|]
+     (right [|1;2;3|] 0) [||]
+   *) (*$= tail & ~printer:(IO.to_string (Array.print Int.print))
+        (tail [|1;2;3|] 1) [|2;3|]
+        [||] (tail [|1;2;3|] 10)
+        (tail [|1;2;3|] 0) [|1;2;3|]
+      *) (*$= head & ~printer:(IO.to_string (Array.print Int.print))
+        (head [|1;2;3|] 1) [|1|]
+        (head [|1;2;3|] 10) [|1;2;3|]
+        (head [|1;2;3|] 0) [||]
+      *)
 
 
 
@@ -748,32 +836,32 @@ struct
   (*BISECT-IGNORE-BEGIN*)
   module Labels =
   struct
-      let init i ~f = init i f
-      let create len ~init = create len init
-      let make = create
-      let make_matrix ~dimx ~dimy x = make_matrix dimx dimy x
-      let create_matrix = make_matrix
-      let sub a ~pos ~len = sub a pos len
-      let fill a ~pos ~len x = fill a pos len x
-      let blit ~src ~src_pos ~dst ~dst_pos ~len = blit src src_pos dst dst_pos len
-      let iter ~f a = iter f a
-      let map ~f a = map  f a
-      let iteri ~f a = iteri f a
-      let mapi ~f a = mapi f a
-      let modify ~f a = modify f a
-      let modifyi ~f a = modifyi f a
-      let fold_left ~f ~init a = fold_left f init a
-      let fold_right ~f a ~init= fold_right f a init
-      let sort ~cmp a = sort cmp a
-      let stable_sort ~cmp a = stable_sort cmp a
-      let fast_sort ~cmp a = fast_sort cmp a
-      let iter2 ~f a b = iter2 f a b
-      let exists ~f a  = exists f a
-      let for_all ~f a = for_all f a
-      let iter2i  ~f a b = iter2i f a b
-      let find ~f a = find f a
-      let filter ~f a = filter f a
-      let filter_map ~f a = filter_map f a
+    let init i ~f = init i f
+    let create len ~init = create len init
+    let make = create
+    let make_matrix ~dimx ~dimy x = make_matrix dimx dimy x
+    let create_matrix = make_matrix
+    let sub a ~pos ~len = sub a pos len
+    let fill a ~pos ~len x = fill a pos len x
+    let blit ~src ~src_pos ~dst ~dst_pos ~len = blit src src_pos dst dst_pos len
+    let iter ~f a = iter f a
+    let map ~f a = map  f a
+    let iteri ~f a = iteri f a
+    let mapi ~f a = mapi f a
+    let modify ~f a = modify f a
+    let modifyi ~f a = modifyi f a
+    let fold_left ~f ~init a = fold_left f init a
+    let fold_right ~f a ~init= fold_right f a init
+    let sort ~cmp a = sort cmp a
+    let stable_sort ~cmp a = stable_sort cmp a
+    let fast_sort ~cmp a = fast_sort cmp a
+    let iter2 ~f a b = iter2 f a b
+    let exists ~f a  = exists f a
+    let for_all ~f a = for_all f a
+    let iter2i  ~f a b = iter2i f a b
+    let find ~f a = find f a
+    let filter ~f a = filter f a
+    let filter_map ~f a = filter_map f a
   end
 
   module Exceptionless =
