@@ -22,10 +22,13 @@
 
 (** Dynamic arrays.
 
-    A dynamic array is equivalent to a OCaml array that will resize itself
+    A dynamic array is equivalent to an OCaml array that will resize itself
     when elements are added or removed, except that floats are boxed and
     that no initialization element is required.
 
+    For all the traversal functions (iter, fold, map, etc.), what happens
+    when the array that is being traversed is mutated is not defined.
+
     @author Brian Hurt
     @author Nicolas Cannasse
     @author David Teller (boilerplate code)
@@ -187,20 +190,20 @@ val keep : ('a -> bool) -> 'a t -> unit
 *)
 
 val filter : ('a -> bool) -> 'a t -> 'a t
-  (** [filter p a] returns all the elements of the array [a]
-      that satisfy the predicate [p].  The order of the elements
-      in the input array is preserved.  
+(** [filter p a] returns all the elements of the array [a]
+    that satisfy the predicate [p].  The order of the elements
+    in the input array is preserved.
+
+    {b Note} This function replaces another function called [filter],
+    available in previous versions of the library. As the old function
+    was incompatible with comprehension of dynamic arrays, its name
+    was changed to {!keep}.
+*)
 
-      {b Note} This function replaces another function called [filter],
-      available in previous versions of the library. As the old function
-      was incompatible with comprehension of dynamic arrays, its name
-      was changed to {!keep}.
-  *)
-  
 val filter_map : ('a -> 'b option) -> 'a t -> 'b t
-  (** [filter_map f e] returns an array consisting in all elements
-      [x] such that [f y] returns [Some x] , where [y] is an element
-      of [e]. *)
+(** [filter_map f e] returns an array consisting of all elements
+    [x] such that [f y] returns [Some x] , where [y] is an element
+    of [e]. *)
 
 
 (** {6 Array resizers} *)
@@ -317,4 +320,7 @@ val unsafe_set : 'a t -> int -> 'a -> unit
 (** {7 Printing}*)
 
 val print :  ?first:string -> ?last:string -> ?sep:string -> ('a BatInnerIO.output -> 'b -> unit) -> 'a BatInnerIO.output -> 'b t -> unit
-val t_printer : 'a BatValue_printer.t -> 'a t BatValue_printer.t
+
+(**/**)
+val invariants : _ t -> unit
+  (**/**)
