@@ -1,4 +1,4 @@
-(* 
+(*
  * File - File manipulation
  * Copyright (C) 2008 David Teller
  *
@@ -20,7 +20,7 @@
 
 (**
    File manipulation.
-   
+
    @author David Teller
 *)
 
@@ -32,11 +32,11 @@ open BatInnerIO
 val lines_of : string -> string BatEnum.t
 (** [line_of name] reads the contents of file [name] as an enumeration of lines.
     The file is automatically closed once the last line has been reached or the
-    enumeration is closed or garbage-collected. *)
+    enumeration is garbage-collected. *)
 
 val write_lines: string -> string BatEnum.t -> unit
 (** [write_lines name lines] writes strings given by [lines] to file [name] with newline character appended to each line. *)
- 
+
 val size_of: string -> int
 (** [size_of name] returns the size of file [name] in bytes.*)
 
@@ -46,7 +46,7 @@ val size_of_big: string -> Int64.t
     This function is provided as the size of a file larger than 1 Gb cannot
     be represented with an [int] on a 32-bit machine.*)
 
-(** {6 File permissions} 
+(** {6 File permissions}
 
     File permissions are used when creating a file to allow controlling which users
     may read, write or open that file. To use a permission, create a value of type
@@ -84,15 +84,15 @@ val group_exec:  permission
    containing the user. Ignored under Windows.*)
 
 val other_read:  permission
-(**Give the permission to read the file to the rest 
+(**Give the permission to read the file to the rest
    of the world. Ignored under Windows.*)
 
 val other_write: permission
-(**Give the permission to modify the file to the rest 
+(**Give the permission to modify the file to the rest
    of the world. Ignored under Windows.*)
 
 val other_exec:  permission
-(**Give the permission to execute the file to the rest 
+(**Give the permission to execute the file to the rest
    of the world. Ignored under Windows.*)
 
 val perm : permission list -> permission
@@ -101,7 +101,8 @@ val perm : permission list -> permission
 val unix_perm : int -> permission
 (**Create a permission from a Unix-style octal integer.
    See your favorite Unix documentation on [chmod]
-   for more details.*)
+   for more details.
+   @raise Invalid_argument if given number outside the [[0, 0o777]] range *)
 
 val set_permissions: string -> permission -> unit
 (** Set the permissions on a file.*)
@@ -115,8 +116,8 @@ type open_in_flag =
   [ `create
   | `excl     (**Fail if the file exists and [`create] is set               *)
   | `text     (**Open in ascii mode -- if this flag is not specified or if the
-		 operating system does not perform conversions, the file is
-		 opened in binary mode.                                     *)
+				 operating system does not perform conversions, the file is
+				 opened in binary mode.                                     *)
   | `nonblock (**Open in non-blocking mode                                  *)
   | `mmap     (**Open in memory-mapped mode (experimental)*)                 ]
 
@@ -131,7 +132,7 @@ val open_in : ?mode:(open_in_flag list) -> ?perm:permission -> string -> input
 
 val with_file_in : ?mode:(open_in_flag list) -> ?perm:permission -> string -> (input -> 'a) -> 'a
 (** [with_file_in file_name f] opens the file named [file_name] for reading,
-    invokes [f] to process the contents of that file then, once [f] has returned 
+    invokes [f] to process the contents of that file then, once [f] has returned
     or triggered an exception, closes the file before proceeding. *)
 
 (** {6 Opening a file for writing} *)
@@ -142,26 +143,26 @@ type open_out_flag =
   | `trunc    (**Empty the file if it already exists; on by default         *)
   | `excl     (**Fail if the file exists and [`create] is set               *)
   | `text     (**Open in ascii mode -- if this flag is not specified or if the
-		 operating system does not perform conversions, the file is
-		 opened in binary mode.                                     *)
+				 operating system does not perform conversions, the file is
+				 opened in binary mode.                                     *)
   | `nonblock (**Open in non-blocking mode                                  *) ]
-    (** Flags governing file output; they correspond to the relevant
-        flags to the POSIX [open()] call.  The default flags are
-        [[`create; `trunc]]. *)
+(** Flags governing file output; they correspond to the relevant
+    flags to the POSIX [open()] call.  The default flags are
+    [[`create; `trunc]]. *)
 
 
 val open_out : ?mode:(open_out_flag list) -> ?perm:permission -> string -> unit output
-  (** [open_out file_name] opens the file named [file_name] for writing.
+(** [open_out file_name] opens the file named [file_name] for writing.
 
-      {b Note} You will need to close the file manually, with
-      {!BatIO.close_out}. An alternative is to call [with_file_out]
-      instead of [open_out].
+    {b Note} You will need to close the file manually, with
+    {!BatIO.close_out}. An alternative is to call [with_file_out]
+    instead of [open_out].
 
-      Naming conventions for files are platform-dependent.*)
+    Naming conventions for files are platform-dependent.*)
 
 val with_file_out: ?mode:(open_out_flag list) -> ?perm:permission -> string -> (unit output -> 'a) -> 'a
 (** [with_file_out file_name f] opens the file named [file_name] for writing,
-    invokes [f] to write onto that file then, once [f] has returned or triggered 
+    invokes [f] to write onto that file then, once [f] has returned or triggered
     an exception, closes the file before proceeding. *)
 
 (** {6 Opening a temporary file for writing} *)
@@ -170,7 +171,7 @@ type open_temporary_out_flag =
   [ open_out_flag
   | `delete_on_exit (**Should the file be deleted when program ends?*) ]
 
-val open_temporary_out: ?mode:(open_temporary_out_flag list) -> ?perm:permission -> ?prefix:string -> ?suffix:string -> unit -> 
+val open_temporary_out: ?mode:(open_temporary_out_flag list) -> ?prefix:string -> ?suffix:string -> ?temp_dir:string -> unit ->
   (unit output * string)
 (** [open_temporary_out ()] opens a new temporary file for writing.
 
@@ -178,6 +179,7 @@ val open_temporary_out: ?mode:(open_temporary_out_flag list) -> ?perm:permission
     (by default ["ocaml"])
     @param suffix a string which should appear at the end of your temporary file name
     (by default ["tmp"])
+    @param temp_dir indicates what temp dir to use
 
     @return The name of the file and the [output] for writing in it.
 
@@ -186,17 +188,17 @@ val open_temporary_out: ?mode:(open_temporary_out_flag list) -> ?perm:permission
 
     Naming conventions for files are platform-dependent.*)
 
-val with_temporary_out: ?mode:(open_temporary_out_flag list) -> ?perm:permission -> ?prefix:string -> ?suffix:string -> (unit output -> string -> 'a) -> 'a
-(** [with_temporary_out f] opens a new temporary file for writing, invokes [f] with
-    to write onto that file then, once [f] has returned or triggered an exception,
-    closes the file before proceeding.
+val with_temporary_out: ?mode:(open_temporary_out_flag list) -> ?prefix:string -> ?suffix:string -> ?temp_dir:string -> (unit output -> string -> 'a) -> 'a
+  (** [with_temporary_out f] opens a new temporary file for writing, invokes [f] with
+      to write onto that file then, once [f] has returned or triggered an exception,
+      closes the file before proceeding.
 
-    @param prefix a string which should appear at the start of your temporary file name
-    (by default ["ocaml"])
-    @param suffix a string which should appear at the end of your temporary file name
-    (by default ["tmp"])
+      @param prefix a string which should appear at the start of your temporary file name
+      (by default ["ocaml"])
+      @param suffix a string which should appear at the end of your temporary file name
+      (by default ["tmp"])
+      @param temp_dir indicates what temp dir to use
 
-    @return The name of the file and the [output] for writing in it.
-
-    Naming conventions for files are platform-dependent.*)
+      @return The name of the file and the [output] for writing in it.
 
+      Naming conventions for files are platform-dependent.*)
