@@ -48,7 +48,7 @@ let input_list ch =
   try loop r
   with
     End_of_file ->
-      Obj.magic r.tl
+    Obj.magic r.tl
 
 let buf_len = 8192
 
@@ -104,17 +104,17 @@ let rec dump r =
     in
     let rec is_list r =
       if Obj.is_int r then
-	r = Obj.repr 0 (* [] *)
+        r = Obj.repr 0 (* [] *)
       else
-	let s = Obj.size r and t = Obj.tag r in
-	t = 0 && s = 2 && is_list (Obj.field r 1) (* h :: t *)
+        let s = Obj.size r and t = Obj.tag r in
+        t = 0 && s = 2 && is_list (Obj.field r 1) (* h :: t *)
     in
     let rec get_list r =
       if Obj.is_int r then
-	[]
+        []
       else
-	let h = Obj.field r 0 and t = get_list (Obj.field r 1) in
-	h :: t
+        let h = Obj.field r 0 and t = get_list (Obj.field r 1) in
+        h :: t
     in
     let opaque name =
       (* XXX In future, print the address of value 'r'.  Not possible
@@ -124,51 +124,51 @@ let rec dump r =
     let s = Obj.size r and t = Obj.tag r in
     (* From the tag, determine the type of block. *)
     match t with
-      | _ when is_list r ->
-	let fields = get_list r in
-	"[" ^ String.concat "; " (List.map dump fields) ^ "]"
-      | 0 ->
-	let fields = get_fields [] s in
-	"(" ^ String.concat ", " (List.map dump fields) ^ ")"
-      | x when x = Obj.lazy_tag ->
-	(* Note that [lazy_tag .. forward_tag] are < no_scan_tag.  Not
-	 * clear if very large constructed values could have the same
-	 * tag. XXX *)
-	opaque "lazy"
-      | x when x = Obj.closure_tag ->
-	opaque "closure"
-      | x when x = Obj.object_tag ->
-	let fields = get_fields [] s in
-	let _clasz, id, slots =
-	  match fields with
-	    | h::h'::t -> h, h', t
-	    | _ -> assert false
-	in
-	(* No information on decoding the class (first field).  So just print
-	 * out the ID and the slots. *)
-	"Object #" ^ dump id ^ " (" ^ String.concat ", " (List.map dump slots) ^ ")"
-      | x when x = Obj.infix_tag ->
-	opaque "infix"
-      | x when x = Obj.forward_tag ->
-	opaque "forward"
-      | x when x < Obj.no_scan_tag ->
-	let fields = get_fields [] s in
-	"Tag" ^ string_of_int t ^
-	  " (" ^ String.concat ", " (List.map dump fields) ^ ")"
-      | x when x = Obj.string_tag ->
-	"\"" ^ String.escaped (Obj.magic r : string) ^ "\""
-      | x when x = Obj.double_tag ->
-	string_of_float (Obj.magic r : float)
-      | x when x = Obj.abstract_tag ->
-	opaque "abstract"
-      | x when x = Obj.custom_tag ->
-	opaque "custom"
-      | x when x = Obj.final_tag ->
-	opaque "final"
-      | x when x = Obj.double_array_tag ->
-	BatIO.to_string (BatArray.print BatFloat.print) (Obj.magic r : float array)
-      | _ ->
-	opaque (Printf.sprintf "unknown: tag %d size %d" t s)
+    | _ when is_list r ->
+      let fields = get_list r in
+      "[" ^ String.concat "; " (List.map dump fields) ^ "]"
+    | 0 ->
+      let fields = get_fields [] s in
+      "(" ^ String.concat ", " (List.map dump fields) ^ ")"
+    | x when x = Obj.lazy_tag ->
+      (* Note that [lazy_tag .. forward_tag] are < no_scan_tag.  Not
+		 * clear if very large constructed values could have the same
+		 * tag. XXX *)
+      opaque "lazy"
+    | x when x = Obj.closure_tag ->
+      opaque "closure"
+    | x when x = Obj.object_tag ->
+      let fields = get_fields [] s in
+      let _clasz, id, slots =
+        match fields with
+        | h::h'::t -> h, h', t
+        | _ -> assert false
+      in
+      (* No information on decoding the class (first field).  So just print
+		 * out the ID and the slots. *)
+      "Object #" ^ dump id ^ " (" ^ String.concat ", " (List.map dump slots) ^ ")"
+    | x when x = Obj.infix_tag ->
+      opaque "infix"
+    | x when x = Obj.forward_tag ->
+      opaque "forward"
+    | x when x < Obj.no_scan_tag ->
+      let fields = get_fields [] s in
+      "Tag" ^ string_of_int t ^
+      " (" ^ String.concat ", " (List.map dump fields) ^ ")"
+    | x when x = Obj.string_tag ->
+      "\"" ^ String.escaped (Obj.magic r : string) ^ "\""
+    | x when x = Obj.double_tag ->
+      string_of_float (Obj.magic r : float)
+    | x when x = Obj.abstract_tag ->
+      opaque "abstract"
+    | x when x = Obj.custom_tag ->
+      opaque "custom"
+    | x when x = Obj.final_tag ->
+      opaque "final"
+    | x when x = Obj.double_array_tag ->
+      BatIO.to_string (BatArray.print BatFloat.print) (Obj.magic r : float array)
+    | _ ->
+      opaque (Printf.sprintf "unknown: tag %d size %d" t s)
 
 let dump v = dump (Obj.repr v)
 
@@ -178,12 +178,12 @@ include BatInnerPervasives
 
 let invisible_args = ref 1
 (* the number or arguments to ignore at the beginning of Sys.argv,
-usually because program-name is put in argv.(0) *)
+   usually because program-name is put in argv.(0) *)
 
 let args () =
   let e = BatArray.enum Sys.argv in
-    BatEnum.drop !invisible_args e;
-    e
+  BatEnum.drop !invisible_args e;
+  e
 
 let exe = Array.get Sys.argv 0
 
@@ -269,7 +269,11 @@ include Infix
 
 (** {6 Operators}*)
 
-let undefined ?(message="Undefined") = failwith message
+let undefined ?(message="Undefined") _ = failwith message
+(*$T undefined
+   ignore (Obj.magic (undefined ~message:"")); true
+   try ignore (undefined ~message:"FooBar" ()); false with Failure "FooBar" -> true
+*)
 
 let verify x ex = if x then () else raise ex
 let verify_arg x s = if x then () else invalid_arg s
@@ -277,4 +281,4 @@ let verify_arg x s = if x then () else invalid_arg s
 (** {6 Clean-up}*)
 
 let _ = at_exit close_all; (*Called second*)
-	at_exit flush_all  (*Called first*)
+  at_exit flush_all  (*Called first*)
