@@ -38,11 +38,11 @@ let length s =
 let rec enum_of_ref r =
   BatEnum.make
     ~next:(fun _ -> match !r () with
-             | Nil ->
-                 raise BatEnum.No_more_elements
-             | Cons(e, s) ->
-                 r := s;
-                 e)
+      | Nil ->
+        raise BatEnum.No_more_elements
+      | Cons(e, s) ->
+        r := s;
+        e)
     ~count:(fun _ -> length !r)
     ~clone:(fun _ -> enum_of_ref (ref !r))
 
@@ -66,20 +66,20 @@ let last s =
     | Cons(e, s) -> aux e s
   in
   match s () with
-    | Nil -> raise (Invalid_argument "Seq.last")
-    | Cons(e, s) -> aux e s
+  | Nil -> raise (Invalid_argument "Seq.last")
+  | Cons(e, s) -> aux e s
 
 let is_empty s = s () = Nil
 
 let at s n =
   let rec aux s n =
     match s () with
-      | Nil -> raise (Invalid_argument "Seq.at")
-      | Cons(e, s) ->
-          if n = 0 then
-            e
-          else
-            aux s (n - 1)
+    | Nil -> raise (Invalid_argument "Seq.at")
+    | Cons(e, s) ->
+      if n = 0 then
+        e
+      else
+        aux s (n - 1)
   in
   if n < 0 then
     invalid_arg "Seq.at"
@@ -93,13 +93,13 @@ let rec append s1 s2 () = match s1 () with
 let concat s =
   let rec aux current rest () = match current () with
     | Cons(e, s) ->
-        Cons(e, aux s rest)
+      Cons(e, aux s rest)
     | Nil ->
-        match rest () with
-          | Cons(e, s) ->
-              aux e s ()
-          | Nil ->
-              Nil
+      match rest () with
+      | Cons(e, s) ->
+        aux e s ()
+      | Nil ->
+        Nil
   in
   aux nil s
 
@@ -129,14 +129,65 @@ let init n f =
   else
     aux 0
 
+let of_list l =
+  let rec aux l () = match l with
+    | [] -> Nil
+    | x::l' -> Cons(x, aux l')
+  in
+  aux l
+
 let rec iter f s = match s () with
   | Nil -> ()
   | Cons(e, s) -> f e; iter f s
 
+let iteri f s =
+  let rec iteri f i s = match s () with
+  | Nil -> ()
+  | Cons(e, s) -> f i e; iteri f (i+1) s
+  in iteri f 0 s
+
+(*$T iteri
+  try iteri (fun i x -> if i<>x then raise Exit) (of_list [0;1;2;3]); true \
+  with Exit -> false
+*)
+
+let rec iter2 f s1 s2 = match s1 (), s2 () with
+  | Nil, _
+  | _, Nil -> ()
+  | Cons (x1, s1'), Cons (x2, s2') -> f x1 x2; iter2 f s1' s2'
+
+(*$T iter2
+  let r = ref 0 in \
+    iter2 (fun i j -> r := !r + i*j) (of_list [1;2]) (of_list [3;2;1]); \
+    !r = 3 + 2*2
+ *)
+
 let rec map f s () = match s () with
   | Nil -> Nil
   | Cons(x, s) -> Cons(f x, map f s)
 
+let mapi f s =
+  let rec mapi f i s () = match s () with
+  | Nil -> Nil
+  | Cons(x, s) -> Cons(f i x, mapi f (i+1) s)
+  in mapi f 0 s
+
+(*$T mapi
+  equal (of_list [0;0;0;0]) \
+    (mapi (fun i x -> i - x) (of_list [0;1;2;3]))
+ *)
+
+let rec map2 f s1 s2 () = match s1 (), s2 () with
+  | Nil, _
+  | _, Nil -> Nil
+  | Cons (x1, s1'), Cons (x2, s2') ->
+    Cons (f x1 x2, map2 f s1' s2')
+
+(*$T map2
+  equal (map2 (+) (of_list [1;2;3]) (of_list [3;2])) \
+    (of_list [4;4])
+ *)
+
 let rec fold_left f acc s = match s () with
   | Nil -> acc
   | Cons(e, s) -> fold_left f (f acc e) s
@@ -157,6 +208,20 @@ let min s = match s () with
   | Nil -> raise (Invalid_argument "Seq.min")
   | Cons(e, s) -> fold_left Pervasives.min e s
 
+let equal ?(eq=(=)) s1 s2 =
+  let rec recurse eq s1 s2 =
+    match s1 (), s2 () with
+    | Nil, Nil -> true
+    | Nil, Cons _
+    | Cons _, Nil -> false
+    | Cons (x1, s1'), Cons (x2, s2') -> eq x1 x2 && recurse eq s1' s2'
+  in
+  recurse eq s1 s2
+
+(*$T of_list
+  equal (of_list [1;2;3]) (nil |> cons 3 |> cons 2 |> cons 1)
+*)
+
 let rec for_all f s = match s () with
   | Nil -> true
   | Cons(e, s) -> f e && for_all f s
@@ -169,41 +234,41 @@ let mem e s = exists ((=) e) s
 
 let rec find f s = match s () with
   | Nil ->
-      None
+    None
   | Cons(e, s) ->
-      if f e then
-        Some e
-      else
-        find f s
+    if f e then
+      Some e
+    else
+      find f s
 
 let rec find_map f s = match s () with
   | Nil ->
-      None
+    None
   | Cons(e, s) ->
-      match f e with
-        | None ->
-            find_map f s
-        | x ->
-            x
+    match f e with
+    | None ->
+      find_map f s
+    | x ->
+      x
 
 let rec filter f s () = match s () with
   | Nil ->
-      Nil
+    Nil
   | Cons(e, s) ->
-      if f e then
-        Cons(e, filter f s)
-      else
-        filter f s ()
+    if f e then
+      Cons(e, filter f s)
+    else
+      filter f s ()
 
 let rec filter_map f s () = match s () with
   | Nil ->
-      Nil
+    Nil
   | Cons(e, s) ->
-      match f e with
-        | None ->
-            filter_map f s ()
-        | Some e ->
-            Cons(e, filter_map f s)
+    match f e with
+    | None ->
+      filter_map f s ()
+    | Some e ->
+      Cons(e, filter_map f s)
 
 let assoc key s = find_map (fun (k, v) -> if k = key then Some v else None) s
 
@@ -212,62 +277,62 @@ let rec take n s () =
     Nil
   else
     match s () with
-      | Nil ->
-          Nil
-      | Cons(e, s) ->
-          Cons(e, take (n - 1) s)
+    | Nil ->
+      Nil
+    | Cons(e, s) ->
+      Cons(e, take (n - 1) s)
 
 let rec drop n s =
   if n <= 0 then
     s
   else
     match s () with
-      | Nil ->
-          nil
-      | Cons(_e, s) ->
-          drop (n - 1) s
+    | Nil ->
+      nil
+    | Cons(_e, s) ->
+      drop (n - 1) s
 
 let rec take_while f s () = match s () with
   | Nil ->
-      Nil
+    Nil
   | Cons(e, s) ->
-      if f e then
-        Cons(e, take_while f s)
-      else
-        Nil
+    if f e then
+      Cons(e, take_while f s)
+    else
+      Nil
 
 let rec drop_while f s = match s () with
   | Nil ->
-      nil
+    nil
   | Cons(e, s) ->
-      if f e then
-        drop_while f s
-      else
-        cons e s
+    if f e then
+      drop_while f s
+    else
+      cons e s
 
 let split s = (map fst s, map snd s)
 
 let rec combine s1 s2 () = match s1 (), s2 () with
   | Nil, Nil ->
-      Nil
+    Nil
   | Cons(e1, s1), Cons(e2, s2) ->
-      Cons((e1, e2), combine s1 s2)
+    Cons((e1, e2), combine s1 s2)
   | _ ->
-      raise (Invalid_argument "Seq.combine")
+    raise (Invalid_argument "Seq.combine")
 
 let print ?(first="[") ?(last="]") ?(sep="; ") print_a out s = match s () with
   | Nil ->
-      BatInnerIO.nwrite out first;
-      BatInnerIO.nwrite out last
+    BatInnerIO.nwrite out first;
+    BatInnerIO.nwrite out last
   | Cons(e, s) ->
-      match s () with
-        | Nil ->
-            BatPrintf.fprintf out "%s%a%s" first print_a e last
-        | _ ->
-            BatInnerIO.nwrite out first;
-            print_a out e;
-            iter (BatPrintf.fprintf out "%s%a" sep print_a) s;
-            BatInnerIO.nwrite out last
+    match s () with
+    | Nil ->
+      BatPrintf.fprintf out "%s%a%s" first print_a e last
+    | _ ->
+      BatInnerIO.nwrite out first;
+      print_a out e;
+      iter (BatPrintf.fprintf out "%s%a" sep print_a) s;
+      BatInnerIO.nwrite out last
 
 module Infix = struct
   (** Infix operators matching those provided by {!BatEnum.Infix} *)
@@ -310,9 +375,9 @@ include Infix
 
 module Exceptionless = struct
   (* This function could be used to eliminate a lot of duplicate code below...
-  let exceptionless_arg f s e =
-    try Some (f s)
-    with Invalid_argument e -> None
+     let exceptionless_arg f s e =
+     try Some (f s)
+     with Invalid_argument e -> None
   *)
 
   let hd s =
@@ -360,4 +425,8 @@ module Exceptionless = struct
   let combine s1 s2 =
     try Some (combine s1 s2)
     with Invalid_argument "Seq.combine" -> None
+
+  (*$T combine
+    equal (combine (of_list [1;2]) (of_list ["a";"b"])) (of_list [1,"a"; 2,"b"])
+  *)
 end
