@@ -54,28 +54,28 @@ module Concrete = struct
     let hr = match r with Empty -> 0 | Node(_,_,_,h) -> h in
     if hl > hr + 2 then begin
       match l with
-          Empty -> invalid_arg "Set.bal"
-        | Node(ll, lv, lr, _) ->
-            if height ll >= height lr then
-              create ll lv (create lr v r)
-            else begin
-              match lr with
-                  Empty -> invalid_arg "Set.bal"
-                | Node(lrl, lrv, lrr, _)->
-                    create (create ll lv lrl) lrv (create lrr v r)
-            end
+        Empty -> invalid_arg "Set.bal"
+      | Node(ll, lv, lr, _) ->
+        if height ll >= height lr then
+          create ll lv (create lr v r)
+        else begin
+          match lr with
+            Empty -> invalid_arg "Set.bal"
+          | Node(lrl, lrv, lrr, _)->
+            create (create ll lv lrl) lrv (create lrr v r)
+        end
     end else if hr > hl + 2 then begin
       match r with
-          Empty -> invalid_arg "Set.bal"
-        | Node(rl, rv, rr, _) ->
-            if height rr >= height rl then
-              create (create l v rl) rv rr
-            else begin
-              match rl with
-                  Empty -> invalid_arg "Set.bal"
-                | Node(rll, rlv, rlr, _) ->
-                    create (create l v rll) rlv (create rlr rv rr)
-            end
+        Empty -> invalid_arg "Set.bal"
+      | Node(rl, rv, rr, _) ->
+        if height rr >= height rl then
+          create (create l v rl) rv rr
+        else begin
+          match rl with
+            Empty -> invalid_arg "Set.bal"
+          | Node(rll, rlv, rlr, _) ->
+            create (create l v rll) rlv (create rlr rv rr)
+        end
     end else
       Node(l, v, r, (if hl >= hr then hl + 1 else hr + 1))
 
@@ -101,36 +101,42 @@ module Concrete = struct
      Assume | height l - height r | <= 2. *)
   let merge t1 t2 =
     match (t1, t2) with
-        (Empty, t) -> t
-      | (t, Empty) -> t
-      | (_, _) -> bal t1 (min_elt t2) (remove_min_elt t2)
+      (Empty, t) -> t
+    | (t, Empty) -> t
+    | (_, _) -> bal t1 (min_elt t2) (remove_min_elt t2)
 
   let pop s =
     match s with
-      | Empty -> raise Not_found
-      | Node (l, v, r, _) ->
-          v, merge l r
+    | Empty -> raise Not_found
+    | Node (l, v, r, _) ->
+      v, merge l r
 
   (* Insertion of one element *)
   let rec add cmp x = function
       Empty -> Node(Empty, x, Empty, 1)
     | Node(l, v, r, _) as t ->
-        let c = cmp x v in
-        if c = 0 then t else
-          if c < 0 then bal (add cmp x l) v r else bal l v (add cmp x r)
+      let c = cmp x v in
+      if c = 0 then t else
+      if c < 0 then bal (add cmp x l) v r else bal l v (add cmp x r)
 
   let rec remove cmp x = function
       Empty -> Empty
     | Node(l, v, r, _) ->
-        let c = cmp x v in
-        if c = 0 then merge l r else
-          if c < 0 then bal (remove cmp x l) v r else bal l v (remove cmp x r)
+      let c = cmp x v in
+      if c = 0 then merge l r else
+      if c < 0 then bal (remove cmp x l) v r else bal l v (remove cmp x r)
 
   let rec mem cmp x = function
       Empty -> false
     | Node(l, v, r, _) ->
-        let c = cmp x v in
-        c = 0 || mem cmp x (if c < 0 then l else r)
+      let c = cmp x v in
+      c = 0 || mem cmp x (if c < 0 then l else r)
+
+  let rec find cmp x = function
+      Empty -> raise Not_found
+    | Node(l, v, r, _) ->
+      let c = cmp x v in
+      if c = 0 then v else  find cmp x (if c < 0 then l else r)
 
   let rec iter f = function
       Empty -> ()
@@ -138,34 +144,38 @@ module Concrete = struct
 
   let rec fold f s accu =
     match s with
-        Empty -> accu
-      | Node(l, v, r, _) -> fold f r (f v (fold f l accu))
+      Empty -> accu
+    | Node(l, v, r, _) -> fold f r (f v (fold f l accu))
 
   let map cmp f s =
     fold (fun v acc -> add cmp (f v) acc) s empty
 
+  let rec op_map f = function
+    | Empty -> Empty
+    | Node (l,x,r,h) -> Node (op_map f l, f x, op_map f r, h)
+
   let singleton x = Node(Empty, x, Empty, 1)
 
   let rec add_min v = function
     | Empty -> singleton v
     | Node (l, x, r, h) ->
-        bal (add_min v l) x r
+      bal (add_min v l) x r
 
   let rec add_max v = function
     | Empty -> singleton v
     | Node (l, x, r, h) ->
-        bal l x (add_max v r)
+      bal l x (add_max v r)
 
   (* Same as create and bal, but no assumptions are made on the
      relative heights of l and r. *)
   let rec join l v r =
     match (l, r) with
-        (Empty, _) -> add_min v r
-      | (_, Empty) -> add_max v l
-      | (Node(ll, lv, lr, lh), Node(rl, rv, rr, rh)) ->
-          if lh > rh + 2 then bal ll lv (join lr v r) else
-            if rh > lh + 2 then bal (join l v rl) rv rr else
-              create l v r
+      (Empty, _) -> add_min v r
+    | (_, Empty) -> add_max v l
+    | (Node(ll, lv, lr, lh), Node(rl, rv, rr, rh)) ->
+      if lh > rh + 2 then bal ll lv (join lr v r) else
+      if rh > lh + 2 then bal (join l v rl) rv rr else
+        create l v r
 
   (* Splitting.  split x s returns a triple (l, present, r) where
      - l is the set of elements of s that are < x
@@ -174,14 +184,76 @@ module Concrete = struct
      or true if s contains an element equal to x. *)
   let rec split cmp x = function
       Empty ->
-        (Empty, false, Empty)
+      (Empty, false, Empty)
     | Node(l, v, r, _) ->
-        let c = cmp x v in
-        if c = 0 then (l, true, r)
-        else if c < 0 then
-          let (ll, pres, rl) = split cmp x l in (ll, pres, join rl v r)
-        else
-          let (lr, pres, rr) = split cmp x r in (join l v lr, pres, rr)
+      let c = cmp x v in
+      if c = 0 then (l, true, r)
+      else if c < 0 then
+        let (ll, pres, rl) = split cmp x l in (ll, pres, join rl v r)
+      else
+        let (lr, pres, rr) = split cmp x r in (join l v lr, pres, rr)
+
+  (* split_opt x s returns a triple (l, maybe_v, r) where
+     - l is the set of elements of s that are < x
+     - r is the set of elements of s that are > x
+     - maybe_v is None if s contains no element equal to x,
+       or (Some v) if s contains an element v that compares equal to x. *)
+  let rec split_opt cmp x = function
+    | Empty -> (Empty, None, Empty)
+    | Node(l, v, r, _) ->
+      let c = cmp x v in
+      if c = 0 then (l, Some v, r)
+      else if c < 0 then
+        let (ll, pres, rl) = split_opt cmp x l in
+        (ll, pres, join rl v r)
+      else (* c > 0 *)
+        let (lr, pres, rr) = split_opt cmp x r in
+        (join l v lr, pres, rr)
+
+  (*$inject
+    let s12    = of_list [1; 2         ] ;;
+    let s45    = of_list [         4; 5] ;;
+    let s1245  = of_list [1; 2;    4; 5] ;;
+    let s12345 = of_list [1; 2; 3; 4; 5] ;;
+  *)
+  (*$T split_opt
+    let l1, mv1, r1 = split_opt 3 s1245 in \
+    (elements l1, mv1, elements r1) = ([1; 2], None  , [4; 5])
+    let l2, mv2, r2 = split_opt 3 s12345 in \
+    (elements l2, mv2, elements r2) = ([1; 2], Some 3, [4; 5])
+  *)
+
+  (* returns a pair of sets: ({y | y < x}, {y | y >= x}) *)
+  let split_lt cmp x s =
+      let l, maybe, r = split_opt cmp x s in
+      match maybe with
+        | None -> l, r
+        | Some eq_x -> l, add cmp eq_x r
+
+  (*$T split_lt
+    let l, r = split_lt 3 s12345 in \
+    (elements l, elements r) = ([1; 2], [3; 4; 5])
+    let l, r = split_lt 3 s12 in \
+    (elements l, elements r) = ([1; 2], [])
+    let l, r = split_lt 3 s45 in \
+    (elements l, elements r) = ([], [4; 5])
+  *)
+
+  (* returns a pair of sets: ({y | y <= x}, {y | y > x}) *)
+  let split_le cmp x s =
+    let l, maybe, r = split_opt cmp x s in
+    match maybe with
+      | None -> l, r
+      | Some eq_x -> add cmp eq_x l, r
+
+  (*$T split_le
+    let l, r = split_le 3 s12345 in \
+    (elements l, elements r) = ([1; 2; 3], [4; 5])
+    let l, r = split_le 3 s12 in \
+    (elements l, elements r) = ([1; 2], [])
+    let l, r = split_le 3 s45 in \
+    (elements l, elements r) = ([], [4; 5])
+  *)
 
   type 'a iter = E | C of 'a * 'a set * 'a iter
 
@@ -193,8 +265,8 @@ module Concrete = struct
       Empty -> accu
     | Node(l, v, r, _) -> elements_aux (v :: elements_aux accu r) l
 
-  let elements s =
-    elements_aux [] s
+  let elements s = elements_aux [] s
+  let to_list = elements
 
   let rec cons_iter s t = match s with
       Empty -> t
@@ -256,104 +328,115 @@ module Concrete = struct
     let rec part (t, f as accu) = function
       | Empty -> accu
       | Node(l, v, r, _) ->
-          part (part (if p v then (add cmp v t, f) else (t, add cmp v f)) l) r in
+        part (part (if p v then (add cmp v t, f) else (t, add cmp v f)) l) r in
     part (Empty, Empty) s
 
   let concat t1 t2 =
     match (t1, t2) with
-        (Empty, t) -> t
-      | (t, Empty) -> t
-      | (_, _) -> join t1 (min_elt t2) (remove_min_elt t2)
+      (Empty, t) -> t
+    | (t, Empty) -> t
+    | (_, _) -> join t1 (min_elt t2) (remove_min_elt t2)
+
+  let cartesian_product a b =
+    let rec product a b = match a with
+      | Empty -> Empty
+      | Node (la, xa, ra, _) ->
+        let lab = product la b in
+        let xab = op_map (fun xb -> (xa,xb)) b in
+        let rab = product ra b in
+        concat lab (concat xab rab)
+    in
+    product a b
 
   let rec union cmp12 s1 s2 =
     match (s1, s2) with
-        (Empty, t2) -> t2
-      | (t1, Empty) -> t1
-      | (Node(l1, v1, r1, h1), Node(l2, v2, r2, h2)) ->
-          if h1 >= h2 then
-            if h2 = 1 then add cmp12 v2 s1 else begin
-              let (l2, _, r2) = split cmp12 v1 s2 in
-              join (union cmp12 l1 l2) v1 (union cmp12 r1 r2)
-            end
-          else
-            if h1 = 1 then add cmp12 v1 s2 else begin
-              let (l1, _, r1) = split cmp12 v2 s1 in
-              join (union cmp12 l1 l2) v2 (union cmp12 r1 r2)
-            end
+      (Empty, t2) -> t2
+    | (t1, Empty) -> t1
+    | (Node(l1, v1, r1, h1), Node(l2, v2, r2, h2)) ->
+      if h1 >= h2 then
+        if h2 = 1 then add cmp12 v2 s1 else begin
+          let (l2, _, r2) = split cmp12 v1 s2 in
+          join (union cmp12 l1 l2) v1 (union cmp12 r1 r2)
+        end
+      else
+      if h1 = 1 then add cmp12 v1 s2 else begin
+        let (l1, _, r1) = split cmp12 v2 s1 in
+        join (union cmp12 l1 l2) v2 (union cmp12 r1 r2)
+      end
 
   let rec sym_diff cmp12 s1 s2 =
     match (s1, s2) with
-        (Empty, t2) -> t2
-      | (t1, Empty) -> t1
-      | (Node(l1, v1, r1, _), t2) ->
-          match split cmp12 v1 t2 with
-              (l2, false, r2) ->
-		join (sym_diff cmp12 l1 l2) v1 (sym_diff cmp12 r1 r2)
-            | (l2, true, r2) ->
-		concat (sym_diff cmp12 l1 l2) (sym_diff cmp12 r1 r2)
+      (Empty, t2) -> t2
+    | (t1, Empty) -> t1
+    | (Node(l1, v1, r1, _), t2) ->
+      match split cmp12 v1 t2 with
+        (l2, false, r2) ->
+        join (sym_diff cmp12 l1 l2) v1 (sym_diff cmp12 r1 r2)
+      | (l2, true, r2) ->
+        concat (sym_diff cmp12 l1 l2) (sym_diff cmp12 r1 r2)
 
   let rec inter cmp12 s1 s2 =
     match (s1, s2) with
-        (Empty, t2) -> Empty
-      | (t1, Empty) -> Empty
-      | (Node(l1, v1, r1, _), t2) ->
-          match split cmp12 v1 t2 with
-              (l2, false, r2) ->
-                concat (inter cmp12 l1 l2) (inter cmp12 r1 r2)
-            | (l2, true, r2) ->
-                join (inter cmp12 l1 l2) v1 (inter cmp12 r1 r2)
+      (Empty, t2) -> Empty
+    | (t1, Empty) -> Empty
+    | (Node(l1, v1, r1, _), t2) ->
+      match split cmp12 v1 t2 with
+        (l2, false, r2) ->
+        concat (inter cmp12 l1 l2) (inter cmp12 r1 r2)
+      | (l2, true, r2) ->
+        join (inter cmp12 l1 l2) v1 (inter cmp12 r1 r2)
 
   let rec diff cmp12 s1 s2 =
     match (s1, s2) with
-        (Empty, t2) -> Empty
-      | (t1, Empty) -> t1
-      | (Node(l1, v1, r1, _), t2) ->
-          match split cmp12 v1 t2 with
-              (l2, false, r2) ->
-                join (diff cmp12 l1 l2) v1 (diff cmp12 r1 r2)
-            | (l2, true, r2) ->
-                concat (diff cmp12 l1 l2) (diff cmp12 r1 r2)
+      (Empty, t2) -> Empty
+    | (t1, Empty) -> t1
+    | (Node(l1, v1, r1, _), t2) ->
+      match split cmp12 v1 t2 with
+        (l2, false, r2) ->
+        join (diff cmp12 l1 l2) v1 (diff cmp12 r1 r2)
+      | (l2, true, r2) ->
+        concat (diff cmp12 l1 l2) (diff cmp12 r1 r2)
 
   let rec disjoint cmp12 s1 s2 =
     match (s1, s2) with
-        (Empty, _)
-      | (_, Empty) -> true
-      | (Node(l1, v1, r1, _), t2) ->
-          match split cmp12 v1 t2 with
-              (l2, false, r2) ->
-                disjoint cmp12 l1 l2 && disjoint cmp12 r1 r2
-            | (l2, true, r2) -> false
+      (Empty, _)
+    | (_, Empty) -> true
+    | (Node(l1, v1, r1, _), t2) ->
+      match split cmp12 v1 t2 with
+        (l2, false, r2) ->
+        disjoint cmp12 l1 l2 && disjoint cmp12 r1 r2
+      | (l2, true, r2) -> false
 
   let compare cmp s1 s2 =
     let rec compare_aux t1' t2' =
       match (t1', t2') with
-          E, E ->  0
-        | E, _ -> -1
-        | _, E ->  1
-        | C (e1, r1, t1), C (e2, r2, t2) ->
-            let c = cmp e1 e2 in
-            if c = 0 then
-              compare_aux (cons_iter r1 t1) (cons_iter r2 t2)
-            else
-              c in
+        E, E ->  0
+      | E, _ -> -1
+      | _, E ->  1
+      | C (e1, r1, t1), C (e2, r2, t2) ->
+        let c = cmp e1 e2 in
+        if c = 0 then
+          compare_aux (cons_iter r1 t1) (cons_iter r2 t2)
+        else
+          c in
     compare_aux (cons_iter s1 E) (cons_iter s2 E)
 
   let equal cmp s1 s2 = compare cmp s1 s2 = 0
 
   let rec subset cmp s1 s2 =
     match (s1, s2) with
-        Empty, _ ->
-          true
-      | _, Empty ->
-          false
-      | Node (l1, v1, r1, _), (Node (l2, v2, r2, _) as t2) ->
-          let c = cmp v1 v2 in
-          if c = 0 then
-            subset cmp l1 l2 && subset cmp r1 r2
-          else if c < 0 then
-            subset cmp (Node (l1, v1, Empty, 0)) l2 && subset cmp r1 t2
-          else
-            subset cmp (Node (Empty, v1, r1, 0)) r2 && subset cmp l1 t2
+      Empty, _ ->
+      true
+    | _, Empty ->
+      false
+    | Node (l1, v1, r1, _), (Node (l2, v2, r2, _) as t2) ->
+      let c = cmp v1 v2 in
+      if c = 0 then
+        subset cmp l1 l2 && subset cmp r1 r2
+      else if c < 0 then
+        subset cmp (Node (l1, v1, Empty, 0)) l2 && subset cmp r1 t2
+      else
+        subset cmp (Node (Empty, v1, r1, 0)) r2 && subset cmp l1 t2
 end
 
 module type S =
@@ -364,6 +447,7 @@ sig
   val is_empty: t -> bool
   val singleton: elt -> t
   val mem: elt -> t -> bool
+  val find: elt -> t -> elt
   val add: elt -> t -> t
   val remove: elt -> t -> t
   val union: t -> t -> t
@@ -384,8 +468,12 @@ sig
   val exists: (elt -> bool) -> t -> bool
   val partition: (elt -> bool) -> t -> t * t
   val split: elt -> t -> t * bool * t
+  val split_opt: elt -> t -> t * elt option * t
+  val split_lt: elt -> t -> t * t
+  val split_le: elt -> t -> t * t
   val cardinal: t -> int
   val elements: t -> elt list
+  val to_list: t -> elt list
   val min_elt: t -> elt
   val max_elt: t -> elt
   val choose: t -> elt
@@ -401,6 +489,7 @@ sig
     val min_elt: t -> elt option
     val max_elt: t -> elt option
     val choose:  t -> elt option
+    val find: elt -> t -> elt option
   end
   (** Operations on {!Set} with labels. *)
   module Labels : sig
@@ -415,7 +504,7 @@ sig
   end
 
 end
-    (** Output signature of the functor {!Set.Make}. *)
+(** Output signature of the functor {!Set.Make}. *)
 
 module Make (Ord : OrderedType) =
 struct
@@ -441,6 +530,7 @@ struct
   let filter f t = t_of_impl (Concrete.filter Ord.compare f (impl_of_t t))
   let filter_map f t = t_of_impl (Concrete.filter_map Ord.compare f (impl_of_t t))
 
+  let find x t = Concrete.find Ord.compare x (impl_of_t t)
   let exists f t = Concrete.exists f (impl_of_t t)
   let for_all f t = Concrete.for_all f (impl_of_t t)
   let paritition f t =
@@ -458,13 +548,30 @@ struct
     let l, v, r = Concrete.split Ord.compare e (impl_of_t s) in
     (t_of_impl l, v, t_of_impl r)
 
+  let split_opt e s =
+    let l, maybe_v, r = Concrete.split_opt Ord.compare e (impl_of_t s) in
+    (t_of_impl l, maybe_v, t_of_impl r)
+
+  let split_lt e s =
+    let l, r = Concrete.split_lt Ord.compare e (impl_of_t s) in
+    (t_of_impl l, t_of_impl r)
+
+  let split_le e s =
+    let l, r = Concrete.split_le Ord.compare e (impl_of_t s) in
+    (t_of_impl l, t_of_impl r)
+
   let singleton e = t_of_impl (Concrete.singleton e)
   let elements t = Concrete.elements (impl_of_t t)
+  let to_list = elements
 
-  let union s1 s2 = t_of_impl (Concrete.union Ord.compare (impl_of_t s1) (impl_of_t s2))
-  let diff s1 s2 = t_of_impl (Concrete.diff Ord.compare (impl_of_t s1) (impl_of_t s2))
-  let inter s1 s2 = t_of_impl (Concrete.inter Ord.compare (impl_of_t s1) (impl_of_t s2))
-  let sym_diff s1 s2 = t_of_impl (Concrete.sym_diff Ord.compare (impl_of_t s1) (impl_of_t s2))
+  let union s1 s2 =
+    t_of_impl (Concrete.union Ord.compare (impl_of_t s1) (impl_of_t s2))
+  let diff s1 s2 =
+    t_of_impl (Concrete.diff Ord.compare (impl_of_t s1) (impl_of_t s2))
+  let inter s1 s2 =
+    t_of_impl (Concrete.inter Ord.compare (impl_of_t s1) (impl_of_t s2))
+  let sym_diff s1 s2 =
+    t_of_impl (Concrete.sym_diff Ord.compare (impl_of_t s1) (impl_of_t s2))
 
   let compare t1 t2 = Concrete.compare Ord.compare (impl_of_t t1) (impl_of_t t2)
   let equal t1 t2 = Concrete.equal Ord.compare (impl_of_t t1) (impl_of_t t2)
@@ -473,20 +580,20 @@ struct
 
   let rec compare_subset s1 s2 =
     match (s1, impl_of_t s2) with
-	(Concrete.Empty, Concrete.Empty) -> 0
-      | (Concrete.Empty, t2) -> -1
-      | (t1, Concrete.Empty) -> 1
-      | (Concrete.Node(l1, v1, r1, _), t2) ->
-          match split v1 (t_of_impl t2) with
-	      (l2, true, r2) -> (* v1 in both s1 and s2 *)
-	        (match compare_subset l1 l2, compare_subset r1 r2 with
-		   | -1, -1 | -1, 0 | 0, -1 -> -1
-		   | 0, 0 -> 0
-		   | 1, 1 | 1, 0 | 0, 1 -> 1
-		   | _ -> min_int)
-            | (l2, false, r2) -> (* v1 in s1, but not in s2 *)
-	        if (compare_subset l1 l2) >= 0 && (compare_subset r1 r2) >= 0
-	        then 1 else min_int
+      (Concrete.Empty, Concrete.Empty) -> 0
+    | (Concrete.Empty, t2) -> -1
+    | (t1, Concrete.Empty) -> 1
+    | (Concrete.Node(l1, v1, r1, _), t2) ->
+      match split v1 (t_of_impl t2) with
+        (l2, true, r2) -> (* v1 in both s1 and s2 *)
+        (match compare_subset l1 l2, compare_subset r1 r2 with
+         | -1, -1 | -1, 0 | 0, -1 -> -1
+         | 0, 0 -> 0
+         | 1, 1 | 1, 0 | 0, 1 -> 1
+         | _ -> min_int)
+      | (l2, false, r2) -> (* v1 in s1, but not in s2 *)
+        if (compare_subset l1 l2) >= 0 && (compare_subset r1 r2) >= 0
+        then 1 else min_int
 
   let compare_subset s1 s2 = compare_subset (impl_of_t s1) s2
 
@@ -498,6 +605,7 @@ struct
     let min_elt t = try Some (min_elt t) with Not_found -> None
     let max_elt t = try Some (max_elt t) with Not_found -> None
     let choose  t = try Some (choose t)  with Not_found -> None
+    let find  e t = try Some (find e t)  with Not_found -> None
   end
 
   module Labels =
@@ -527,14 +635,15 @@ module PSet = struct (*$< PSet *)
   let create cmp  = { cmp = cmp; set = Concrete.empty }
   let get_cmp {cmp} = cmp
 
-(*$T get_cmp
-  get_cmp (create Int.compare) == Int.compare
-*)
+  (*$T get_cmp
+    get_cmp (create Int.compare) == Int.compare
+  *)
 
 
   let singleton ?(cmp = compare) x = { cmp = cmp; set = Concrete.singleton x }
   let is_empty s = Concrete.is_empty s.set
   let mem x s = Concrete.mem s.cmp x s.set
+  let find x s = Concrete.find s.cmp x s.set
   let add x s  = { s with set = Concrete.add s.cmp x s.set }
   let remove x s = { s with set = Concrete.remove s.cmp x s.set }
   let iter f s = Concrete.iter f s.set
@@ -547,6 +656,7 @@ module PSet = struct (*$< PSet *)
   let exists f s = Concrete.exists f s.set
   let cardinal s = fold (fun _ acc -> acc + 1) s 0
   let elements s = Concrete.elements s.set
+  let to_list = elements
   let choose s = Concrete.choose s.set
   let min_elt s = Concrete.min_elt s.set
   let max_elt s = Concrete.max_elt s.set
@@ -566,6 +676,15 @@ module PSet = struct (*$< PSet *)
   let split e s =
     let s1, found, s2 = Concrete.split s.cmp e s.set in
     { s with set = s1 }, found, { s with set = s2 }
+  let split_opt e s =
+    let s1, maybe_v, s2 = Concrete.split_opt s.cmp e s.set in
+    { s with set = s1 }, maybe_v, { s with set = s2 }
+  let split_lt e s =
+    let s1, s2 = Concrete.split_lt s.cmp e s.set in
+    { s with set = s1 }, {s with set = s2 }
+  let split_le e s =
+    let s1, s2 = Concrete.split_le s.cmp e s.set in
+    { s with set = s1 }, {s with set = s2 }
   let union s1 s2 =
     { s1 with set = Concrete.union s1.cmp s1.set s2.set }
   let diff s1 s2 =
@@ -593,6 +712,19 @@ let is_empty s = s = Concrete.Empty
 
 let mem x s = Concrete.mem Pervasives.compare x s
 
+let find x s = Concrete.find Pervasives.compare x s
+
+(*$T
+  (find 1 (of_list [1;2;3;4;5;6;7;8])) == 1
+  (find 8 (of_list [1;2;3;4;5;6;7;8])) == 8
+  (find 1 (singleton 1)) == 1
+  let x = "abc" in (find "abc" (singleton x)) == x
+  let x = (1,1) in (find (1,1) (singleton x)) == x
+  let x,y = (1,1),(1,1) in find x (singleton y) == y
+  let x,y = "a","a" in find x (singleton y) != x
+  try ignore (find (1,2) (singleton (1,1))); false with Not_found -> true
+*)
+
 let add x s  = Concrete.add Pervasives.compare x s
 
 let remove x s = Concrete.remove Pervasives.compare x s
@@ -616,6 +748,7 @@ let exists f s = Concrete.exists f s
 let cardinal s = fold (fun _ acc -> acc + 1) s 0
 
 let elements s = Concrete.elements s
+let to_list = elements
 
 let choose s = Concrete.choose s
 
@@ -659,7 +792,11 @@ let print ?first ?last ?sep print_elt out s =
 let for_all f s = Concrete.for_all f s
 let partition f s = Concrete.partition Pervasives.compare f s
 let pop s = Concrete.pop s
+let cartesian_product = Concrete.cartesian_product
 let split e s = Concrete.split Pervasives.compare e s
+let split_opt e s = Concrete.split_opt Pervasives.compare e s
+let split_lt e s = Concrete.split_lt Pervasives.compare e s
+let split_le e s = Concrete.split_le Pervasives.compare e s
 let union s1 s2 = Concrete.union Pervasives.compare s1 s2
 let diff s1 s2 = Concrete.diff Pervasives.compare s1 s2
 let sym_diff s1 s2 = Concrete.sym_diff Pervasives.compare s1 s2
@@ -673,7 +810,7 @@ let disjoint s1 s2 = Concrete.disjoint Pervasives.compare s1 s2
    subset (of_list [1;2;3]) (of_list [1;2;3;4])
    not (subset (of_list [1;2;3;5]) (of_list [1;2;3;4]))
    not (subset (of_list [1;2;3;4]) (of_list [1;2;3]))
- *)
+*)
 
 (*$T compare
   compare (of_list [1;2;3]) (of_list [1;2;3;4]) <> 0
@@ -681,4 +818,26 @@ let disjoint s1 s2 = Concrete.disjoint Pervasives.compare s1 s2
   let a = of_list [1;2;3] and b = of_list [1;2;3;4] and c = of_list [3;1;2] in\
     compare a b = - (compare b c)
   compare (of_list [1;2;3]) (of_list [3;1;2]) = 0
+*)
+
+(*$T cartesian_product
+  cartesian_product (of_list [1;2;3]) (of_list ["a"; "b"]) |> to_list = \
+    [1, "a"; 1, "b"; 2, "a"; 2, "b"; 3, "a"; 3, "b"]
+  is_empty @@ cartesian_product (of_list [1;2;3]) empty
+  is_empty @@ cartesian_product empty (of_list [1;2;3])
+  let s1, s2 = of_list ["a"; "b"; "c"], of_list [1;2;3] in \
+    equal (cartesian_product s1 s2) \
+          (map BatTuple.Tuple2.swap (cartesian_product s2 s1))
  *)
+
+
+module Incubator = struct (*$< Incubator *)
+  let op_map f s = Concrete.op_map f s
+    (*$T op_map
+      of_enum (1--3) |> op_map ((+) 2) |> mem 5
+      of_enum (1--3) |> op_map ((+) 2) |> mem 4
+      of_enum (1--3) |> op_map ((+) 2) |> mem 3
+    *)
+
+
+end (*$>*)
