@@ -462,6 +462,11 @@ val rfind_from: string -> int -> string -> int
     Example: [String.rfind_from "foobarbaz" 6 "ba" = 6]
 *)
 
+val find_all : string -> string -> int BatEnum.t
+(** [find_all s x] enumerates positions of [s] at which [x] occurs.
+    Example: [find_all "aabaabaa" "aba" |> List.of_enum] will return
+    the list [[1; 4]].
+    @since 2.2.0 *)
 
 val ends_with : string -> string -> bool
 (** [ends_with s x] returns [true] if the string [s] is ending with [x], [false] otherwise.
@@ -743,6 +748,12 @@ module NumString : BatInterfaces.OrderedType with type t = t
     Example: [module FilenameSet = Set.Make(String.NumString)]
 *)
 
+val edit_distance : t -> t -> int
+(** Edition distance (also known as "Levenshtein distance").
+    See {{:http://en.wikipedia.org/wiki/Levenshtein_distance} wikipedia}
+    @since 2.2.0
+*)
+
 (** {6 Boilerplate code}*)
 
 (** {7 Printing}*)
