@@ -1,7 +1,6 @@
 open OUnit
 open BatFile
 open BatIO
-open BatPrint
 open BatPervasives
 
 (**Initialize data sample*)
@@ -11,7 +10,7 @@ let buffer = BatArray.of_enum (BatEnum.take 60 (BatRandom.State.enum_int state 2
 (**Write sample to temporary file*)
 let write buf =
   let (out, name) = open_temporary_out ~mode:[`delete_on_exit] () in
-    write_bytes out (BatArray.enum buf);
+    BatEnum.iter (write_byte out) (BatArray.enum buf);
     close_out out;
     name
 
@@ -69,8 +68,9 @@ let test_open_close_many_pervasives () =
     for i = 0 to 10000 do
       let temp = temp_file "batteries" "test" in
       let oc   = open_out temp                         in
-        output_string oc "test";
-        close_out oc
+      output_string oc "test";
+      close_out oc;
+      (try Unix.unlink temp with _ -> ())
     done;
     (* pass *)
   with Sys_error e -> assert_failure (BatPrintf.sprintf "Got Sys_error %S" e)
@@ -79,11 +79,11 @@ let test_no_append () =
   try
     let temp   = temp_file "ocaml_batteries" "noappend_test" in
     let out    = open_out temp                       in
-    let _      = write_bytes out (BatArray.enum buffer) in
+    let _      = BatEnum.iter (write_byte out) (BatArray.enum buffer) in
     let _      = close_out out                       in
     let size_1 = size_of temp                        in
     let out    = open_out temp                       in
-    let _      = write_bytes out (BatArray.enum buffer) in
+    let _      = BatEnum.iter (write_byte out) (BatArray.enum buffer) in
     let _      = close_out out                       in
     let size_2 = size_of temp                        in
       if size_1 <> size_2 then assert_failure
@@ -94,23 +94,43 @@ let test_append () =
   try
     let temp   = temp_file "ocaml_batteries" "append_test" in
     let out    = open_out ~mode:[`append] temp       in
-    let _      = write_bytes out (BatArray.enum buffer) in
+    let _      = BatEnum.iter (write_byte out) (BatArray.enum buffer) in
     let _      = close_out out                       in
     let size_1 = size_of temp                        in
     let out    = open_out ~mode:[`append] temp       in
-    let _      = write_bytes out (BatArray.enum buffer) in
+    let _      = BatEnum.iter (write_byte out) (BatArray.enum buffer) in
     let _      = close_out out                       in
     let size_2 = size_of temp                        in
       if size_2 <> 2*size_1 then assert_failure
-	(BatPrintf.sprintf "Expected a files with size %d, got a first chunk with size %d and a second chunk with size %d" 
+	(BatPrintf.sprintf "Expected a files with size %d, got a first chunk with size %d and a second chunk with size %d"
 	   (2*size_1) size_1 size_2)
   with Sys_error e -> assert_failure (BatPrintf.sprintf "Got Sys_error %S" e)
 
+let test_lines_of () =
+  let file_lines_of fn =
+    let ic = Pervasives.open_in fn in
+    BatEnum.suffix_action
+      (fun () -> Pervasives.close_in ic)
+      (BatEnum.from (fun () -> try Pervasives.input_line ic with End_of_file -> raise BatEnum.No_more_elements))
+  in
+  try
+    let open Batteries in
+    let tf = temp_file "batteries" "test" in
+    BatFile.write_lines tf (BatList.enum [ "First" ; "Second" ]) ;
+    (file_lines_of tf
+       /@ (fun x -> String.length x, 42)
+       |> Enum.group Tuple2.first)
+    |> List.of_enum
+    |> ignore
+  with Sys_error e -> assert_failure (BatPrintf.sprintf "Got Sys_error %S" e)
+
+
 let tests = "File" >::: [
   "Reading back output to temporary file" >:: test_read_back_tmp;
   "open_in'd files should not autoclose" >:: test_open_files_not_autoclosed;
   "opening and closing many files" >:: test_open_close_many;
   "opening and closing many files (Pervasives)" >:: test_open_close_many_pervasives;
   "default truncation of files" >:: test_no_append;
-  "appending to a file" >:: test_append
+  "appending to a file" >:: test_append;
+  "reading lines of a file" >:: test_lines_of
 ]
